package com.sun.net.httpserver.spi;

import java.security.AccessController;
import sun.net.httpserver.DefaultHttpServerProvider;
import java.security.PrivilegedAction;
import java.util.Iterator;
import java.util.ServiceLoader;
import java.util.ServiceConfigurationError;
import com.sun.net.httpserver.HttpsServer;
import java.io.IOException;
import com.sun.net.httpserver.HttpServer;
import java.net.InetSocketAddress;
import jdk.Exported;

@Exported
public abstract class HttpServerProvider
{
    private static final Object lock;
    private static HttpServerProvider provider;
    
    public abstract HttpServer createHttpServer(InetSocketAddress p0, int p1) throws IOException;
    
    public abstract HttpsServer createHttpsServer(InetSocketAddress p0, int p1) throws IOException;
    
    protected HttpServerProvider() {
        SecurityManager securityManager = System.getSecurityManager();
        if (securityManager != null) {
            securityManager.checkPermission(new RuntimePermission("httpServerProvider"));
        }
    }
    
    private static boolean loadProviderFromProperty() {
        String property = System.getProperty("com.sun.net.httpserver.HttpServerProvider");
        if (property == null) {
            return false;
        }
        try {
            HttpServerProvider.provider = (HttpServerProvider)Class.forName(property, true, ClassLoader.getSystemClassLoader()).newInstance();
            return true;
        }
        catch (ClassNotFoundException | IllegalAccessException | InstantiationException | SecurityException cause) {
            throw new ServiceConfigurationError(null, cause);
        }
    }
    
    private static boolean loadProviderAsService() {
        Iterator<HttpServerProvider> iterator = ServiceLoader.load(HttpServerProvider.class, ClassLoader.getSystemClassLoader()).iterator();
        Label_0012: {
            try {
                if (!iterator.hasNext()) {
                    return false;
                }
                HttpServerProvider.provider = (HttpServerProvider)iterator.next();
                return true;
            }
            catch (ServiceConfigurationError serviceConfigurationError) {
                if ((serviceConfigurationError.getCause() instanceof SecurityException)) {
                    continue;
                }
                throw serviceConfigurationError;
            }
        }
    }
    
    public static HttpServerProvider provider() {
        synchronized (HttpServerProvider.lock) {
            if (HttpServerProvider.provider != null) {
                return HttpServerProvider.provider;
            }
            return (HttpServerProvider)AccessController.doPrivileged(new PrivilegedAction<Object>() {
                @Override
                public Object run() {
                    if (HttpServerProvider.access$000()) {
                        return HttpServerProvider.provider;
                    }
                    if (HttpServerProvider.access$200()) {
                        return HttpServerProvider.provider;
                    }
                    HttpServerProvider.provider = new DefaultHttpServerProvider();
                    return HttpServerProvider.provider;
                }
            });
        }
    }
    
    static boolean access$000() {
        return loadProviderFromProperty();
    }
    
    static boolean access$200() {
        return loadProviderAsService();
    }
    
    static {
        lock = new Object();
        HttpServerProvider.provider = null;
    }
}
